package net.voltexstudios.entity.client.bandit;

import net.minecraft.client.model.*;
import net.minecraft.client.render.VertexConsumer;
import net.minecraft.client.render.entity.model.CrossbowPosing;
import net.minecraft.client.render.entity.model.EntityModelLayer;
import net.minecraft.client.render.entity.model.ModelWithArms;
import net.minecraft.client.render.entity.model.SinglePartEntityModel;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.entity.mob.IllagerEntity;
import net.minecraft.util.Arm;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.MathHelper;
import net.voltexstudios.VoltexDesertUpdate;
import net.voltexstudios.entity.custom.BanditEntity;

public class BanditModel<T extends BanditEntity> extends SinglePartEntityModel<T> implements ModelWithArms {

    public static final EntityModelLayer BANDIT = new EntityModelLayer(Identifier.of(VoltexDesertUpdate.MOD_ID, "bandit"), "main");

    private final ModelPart mambo;
    private final ModelPart head;
    private final ModelPart body;
    private final ModelPart arms;
    private final ModelPart right_leg;
    private final ModelPart left_leg;
    private final ModelPart right_arm;
    private final ModelPart left_arm;

    public BanditModel(ModelPart root) {
        this.mambo = root.getChild("mambo");
        this.head = this.mambo.getChild("head");
        this.body = this.mambo.getChild("body");
        this.arms = this.mambo.getChild("arms");
        this.right_leg = this.mambo.getChild("right_leg");
        this.left_leg = this.mambo.getChild("left_leg");
        this.right_arm = this.mambo.getChild("right_arm");
        this.left_arm = this.mambo.getChild("left_arm");
    }
    public static TexturedModelData getTexturedModelData() {
        ModelData modelData = new ModelData();
        ModelPartData modelPartData = modelData.getRoot();
        ModelPartData mambo = modelPartData.addChild("mambo", ModelPartBuilder.create(), ModelTransform.pivot(0.0F, 24.0F, 0.0F));

        ModelPartData head = mambo.addChild("head", ModelPartBuilder.create().uv(28, 18).cuboid(-4.0F, -10.0F, -4.0F, 8.0F, 10.0F, 8.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, -24.0F, 0.0F));

        ModelPartData nose = head.addChild("nose", ModelPartBuilder.create().uv(52, 36).cuboid(-1.0F, -1.0F, -6.0F, 2.0F, 4.0F, 2.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, -2.0F, 0.0F));

        ModelPartData headwear = head.addChild("headwear", ModelPartBuilder.create().uv(64, 0).cuboid(-4.0F, -10.0F, -4.0F, 8.0F, 10.0F, 8.0F, new Dilation(0.51F)), ModelTransform.pivot(0.0F, 0.0F, 0.0F));

        ModelPartData headwear2 = head.addChild("headwear2", ModelPartBuilder.create().uv(0, 0).cuboid(-7.0F, -7.0F, -8.3F, 14.0F, 14.0F, 4.0F, new Dilation(0.0F))
                .uv(0, 0).cuboid(-7.0F, -7.0F, -8.3F, 14.0F, 14.0F, 4.0F, new Dilation(0.0F)), ModelTransform.of(0.0F, 0.0F, 0.0F, -1.5708F, 0.0F, 0.0F));

        ModelPartData body = mambo.addChild("body", ModelPartBuilder.create().uv(36, 0).cuboid(-4.0F, 0.0F, -3.0F, 8.0F, 12.0F, 6.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, -24.0F, 0.0F));

        ModelPartData bodywear = body.addChild("bodywear", ModelPartBuilder.create().uv(0, 18).cuboid(-4.0F, 0.0F, -3.0F, 8.0F, 20.0F, 6.0F, new Dilation(0.5F)), ModelTransform.pivot(0.0F, 0.0F, 0.0F));

        ModelPartData arms = mambo.addChild("arms", ModelPartBuilder.create().uv(28, 36).cuboid(-4.0F, 2.0F, -2.0F, 8.0F, 4.0F, 4.0F, new Dilation(0.0F))
                .uv(32, 44).cuboid(-8.0F, -2.0F, -2.0F, 4.0F, 8.0F, 4.0F, new Dilation(0.0F)), ModelTransform.of(0.0F, -21.05F, -1.05F, -0.7505F, 0.0F, 0.0F));

        ModelPartData mirrored = arms.addChild("mirrored", ModelPartBuilder.create().uv(48, 44).cuboid(4.0F, -23.05F, -3.05F, 4.0F, 8.0F, 4.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, 21.05F, 1.05F));

        ModelPartData right_leg = mambo.addChild("right_leg", ModelPartBuilder.create().uv(0, 44).cuboid(-2.0F, 0.0F, -2.0F, 4.0F, 12.0F, 4.0F, new Dilation(0.0F)), ModelTransform.pivot(-2.0F, -12.0F, 0.0F));

        ModelPartData left_leg = mambo.addChild("left_leg", ModelPartBuilder.create().uv(16, 44).cuboid(-2.0F, 0.0F, -2.0F, 4.0F, 12.0F, 4.0F, new Dilation(0.0F)), ModelTransform.pivot(2.0F, -12.0F, 0.0F));

        ModelPartData right_arm = mambo.addChild("right_arm", ModelPartBuilder.create().uv(0, 60).cuboid(-3.0F, -2.0F, -2.0F, 4.0F, 12.0F, 4.0F, new Dilation(0.0F)), ModelTransform.pivot(-5.0F, -22.0F, 0.0F));

        ModelPartData left_arm = mambo.addChild("left_arm", ModelPartBuilder.create().uv(16, 60).mirrored().cuboid(-1.0F, -2.0F, -2.0F, 4.0F, 12.0F, 4.0F, new Dilation(0.0F)).mirrored(false), ModelTransform.pivot(5.0F, -22.0F, 0.0F));
        return TexturedModelData.of(modelData, 128, 128);
    }

    @Override
    public void setAngles(BanditEntity banditEntity, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch) {
        this.head.yaw = netHeadYaw * (float) (Math.PI / 180.0);
        this.head.pitch = headPitch * (float) (Math.PI / 180.0);
        if (this.riding) {
            this.right_arm.pitch = (float) (-Math.PI / 5);
            this.right_arm.yaw = 0.0F;
            this.right_arm.roll = 0.0F;
            this.left_arm.pitch = (float) (-Math.PI / 5);
            this.left_arm.yaw = 0.0F;
            this.left_arm.roll = 0.0F;
            this.right_leg.pitch = -1.4137167F;
            this.right_leg.yaw = (float) (Math.PI / 10);
            this.right_leg.roll = 0.07853982F;
            this.left_leg.pitch = -1.4137167F;
            this.left_leg.yaw = (float) (-Math.PI / 10);
            this.left_leg.roll = -0.07853982F;
        } else {
            this.right_arm.pitch = MathHelper.cos(limbSwing * 0.6662F + (float) Math.PI) * 2.0F * limbSwingAmount * 0.5F;
            this.right_arm.yaw = 0.0F;
            this.right_arm.roll = 0.0F;
            this.left_arm.pitch = MathHelper.cos(limbSwing * 0.6662F) * 2.0F * limbSwingAmount * 0.5F;
            this.left_arm.yaw = 0.0F;
            this.left_arm.roll = 0.0F;
            this.right_leg.pitch = MathHelper.cos(limbSwing * 0.6662F) * 1.4F * limbSwingAmount * 0.5F;
            this.right_leg.yaw = 0.0F;
            this.right_leg.roll = 0.0F;
            this.left_leg.pitch = MathHelper.cos(limbSwing * 0.6662F + (float) Math.PI) * 1.4F * limbSwingAmount * 0.5F;
            this.left_leg.yaw = 0.0F;
            this.left_leg.roll = 0.0F;
        }

        IllagerEntity.State state = banditEntity.getState();
        if (state == IllagerEntity.State.ATTACKING) {
            if (banditEntity.getMainHandStack().isEmpty()) {
                CrossbowPosing.meleeAttack(this.left_arm, this.right_arm, true, this.handSwingProgress, ageInTicks);
            } else {
                CrossbowPosing.meleeAttack(this.right_arm, this.left_arm, banditEntity, this.handSwingProgress, ageInTicks);
            }
        } else if (state == IllagerEntity.State.CROSSBOW_CHARGE) {
            CrossbowPosing.charge(this.right_arm, this.left_arm, banditEntity, true);
        } else if (state == IllagerEntity.State.CELEBRATING) {
            this.right_arm.pivotZ = 0.0F;
            this.right_arm.pivotX = -5.0F;
            this.right_arm.pitch = MathHelper.cos(ageInTicks * 0.6662F) * 0.05F;
            this.right_arm.roll = 2.670354F;
            this.right_arm.yaw = 0.0F;
            this.left_arm.pivotZ = 0.0F;
            this.left_arm.pivotX = 5.0F;
            this.left_arm.pitch = MathHelper.cos(ageInTicks * 0.6662F) * 0.05F;
            this.left_arm.roll = (float) (-Math.PI * 3.0 / 4.0);
            this.left_arm.yaw = 0.0F;
        }

        boolean bl = state == IllagerEntity.State.CROSSED;
        this.arms.visible = bl;
        this.left_arm.visible = !bl;
        this.right_arm.visible = !bl;
    }

    @Override
    public void render(MatrixStack matrices, VertexConsumer vertexConsumer, int light, int overlay, int color) {
        mambo.render(matrices, vertexConsumer, light, overlay, color);
    }

    private ModelPart getAttackingArm(Arm arm) {
        return arm == Arm.LEFT ? this.left_arm : this.right_arm;
    }

    @Override
    public ModelPart getPart() {
        return mambo;
    }

    @Override
    public void setArmAngle(Arm arm, MatrixStack matrices) {
        this.getAttackingArm(arm).rotate(matrices);
    }
}