package net.voltexstudios.entity.client.cobra;

import net.minecraft.client.model.*;
import net.minecraft.client.render.VertexConsumer;
import net.minecraft.client.render.entity.model.EntityModelLayer;
import net.minecraft.client.render.entity.model.SinglePartEntityModel;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.MathHelper;
import net.voltexstudios.VoltexDesertUpdate;
import net.voltexstudios.entity.custom.CobraEntity;

public class CobraModel<T extends CobraEntity> extends SinglePartEntityModel<T> {

    public static final EntityModelLayer COBRA = new EntityModelLayer(Identifier.of(VoltexDesertUpdate.MOD_ID, "cobra"), "main");

    private final ModelPart cobra;
    private final ModelPart head;
    private final ModelPart body;

    public CobraModel(ModelPart root) {
        this.cobra = root.getChild("mambo");
        this.head = this.cobra.getChild("head");
        this.body = this.cobra.getChild("body");
    }

    public static TexturedModelData getTexturedModelData() {
        ModelData modelData = new ModelData();
        ModelPartData modelPartData = modelData.getRoot();
        ModelPartData mambo = modelPartData.addChild("mambo", ModelPartBuilder.create(), ModelTransform.pivot(0.5F, 14.0F, -13.0F));

        ModelPartData head = mambo.addChild("head", ModelPartBuilder.create().uv(20, 9).cuboid(-1.5F, -9.125F, -2.125F, 3.0F, 10.0F, 3.0F, new Dilation(0.0F))
                .uv(20, 22).cuboid(-4.0F, -12.125F, -0.125F, 8.0F, 8.0F, 0.0F, new Dilation(0.0F)), ModelTransform.pivot(-0.5F, 9.125F, 6.125F));

        ModelPartData head_look = head.addChild("head_look", ModelPartBuilder.create().uv(0, 20).cuboid(-2.0F, -2.8333F, -4.3333F, 4.0F, 3.0F, 6.0F, new Dilation(0.0F))
                .uv(32, 20).cuboid(-1.5F, 0.1667F, -4.3333F, 0.0F, 1.0F, 1.0F, new Dilation(0.0F))
                .uv(32, 19).cuboid(1.5F, 0.1667F, -4.3333F, 0.0F, 1.0F, 1.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, -9.2917F, -0.7917F));

        ModelPartData tounge = head_look.addChild("tounge", ModelPartBuilder.create().uv(0, 29).cuboid(-1.5F, 0.0F, -3.0F, 3.0F, 0.0F, 3.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, 0.1667F, -4.3333F));

        ModelPartData body = mambo.addChild("body", ModelPartBuilder.create().uv(0, 0).cuboid(-2.0F, -2.0F, 0.0F, 4.0F, 4.0F, 6.0F, new Dilation(0.0F)), ModelTransform.pivot(-0.5F, 8.0F, 7.0F));

        ModelPartData bone = body.addChild("bone", ModelPartBuilder.create().uv(0, 10).cuboid(-2.0F, -2.0F, 0.0F, 4.0F, 4.0F, 6.0F, new Dilation(0.01F)), ModelTransform.pivot(0.0F, 0.0F, 6.0F));

        ModelPartData bone2 = bone.addChild("bone2", ModelPartBuilder.create().uv(20, 0).cuboid(-1.0F, -1.0F, 0.0F, 2.0F, 2.0F, 7.0F, new Dilation(0.0F)), ModelTransform.pivot(0.0F, 1.0F, 6.0F));
        return TexturedModelData.of(modelData, 64, 64);
    }

    @Override
    public void setAngles(CobraEntity entity, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch) {
        this.getPart().traverse().forEach(ModelPart::resetTransform);
        this.setHeadAngles(netHeadYaw, headPitch);

        this.animateMovement(CobraAnimations.MOVE, limbSwing, limbSwingAmount, 2f, 2.5f);
        this.updateAnimation(entity.idleAnimationState, CobraAnimations.IDLE, ageInTicks, 1f);
    }

    private void setHeadAngles(float headYaw, float headPitch) {
        headYaw = MathHelper.clamp(headYaw, -30.0F, 30.0F);
        headPitch = MathHelper.clamp(headPitch, -25.0F, 45.0F);

        this.head.yaw = headYaw * 0.017453292F;
        this.head.pitch = headPitch * 0.017453292F;
    }

    @Override
    public void render(MatrixStack matrices, VertexConsumer vertexConsumer, int light, int overlay, int color) {
        cobra.render(matrices, vertexConsumer, light, overlay, color);
    }

    @Override
    public ModelPart getPart() {
        return cobra;
    }
}
