package net.voltexstudios.entity.custom;

import net.minecraft.block.BlockState;
import net.minecraft.entity.AnimationState;
import net.minecraft.entity.EntityType;
import net.minecraft.entity.ai.goal.*;
import net.minecraft.entity.attribute.DefaultAttributeContainer;
import net.minecraft.entity.attribute.EntityAttributes;
import net.minecraft.entity.mob.HostileEntity;
import net.minecraft.entity.mob.MobEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.shape.VoxelShapes;
import net.minecraft.world.World;
import net.voltexstudios.blocks.ModBlocks;

import java.util.EnumSet;

public class CobraEntity extends HostileEntity {

    public final AnimationState idleAnimationState = new AnimationState();
    private int idleAnimationTimeout = 0;

    public CobraEntity(EntityType<? extends CobraEntity> entityType, World world) {
        super(entityType, world);
    }

    @Override
    protected void initGoals() {
        this.goalSelector.add(1, new SwimGoal(this));
        this.goalSelector.add(1, new CobraEntity.QuicksandJumpGoal(this, this.getWorld()));
        this.goalSelector.add(4, new MeleeAttackGoal(this, 0.8, false));
        this.goalSelector.add(5, new WanderAroundGoal(this, 0.8));

        this.targetSelector.add(1, new RevengeGoal(this).setGroupRevenge());
        this.targetSelector.add(2, new ActiveTargetGoal<>(this, PlayerEntity.class, true));
    }

    @Override
    public void tick() {
        super.tick();

        if (this.getWorld().isClient()) {
            this.setupAnimationStates();
        }
    }

    private void setupAnimationStates() {
        if (this.idleAnimationTimeout <= 0) {
            this.idleAnimationTimeout = 14;
            this.idleAnimationState.start(this.age);
        } else {
            --this.idleAnimationTimeout;
        }
    }

    public static DefaultAttributeContainer.Builder createCobraAttributes() {
        return HostileEntity.createHostileAttributes()
                .add(EntityAttributes.GENERIC_MAX_HEALTH, 8.0)
                .add(EntityAttributes.GENERIC_MOVEMENT_SPEED, 0.4f)
                .add(EntityAttributes.GENERIC_ATTACK_DAMAGE, 1.0);
    }

    static class QuicksandJumpGoal extends Goal {
        private final MobEntity entity;
        private final World world;

        public QuicksandJumpGoal(MobEntity entity, World world) {
            this.entity = entity;
            this.world = world;
            this.setControls(EnumSet.of(Goal.Control.JUMP));
        }

        @Override
        public boolean canStart() {
            boolean bl = this.entity.hurtTime > 0;
            if (bl) {
                BlockPos blockPos = this.entity.getBlockPos().up();
                BlockState blockState = this.world.getBlockState(blockPos);
                return blockState.isOf(ModBlocks.QUICKSAND_BLOCK) || blockState.getCollisionShape(this.world, blockPos) == VoxelShapes.empty();
            } else {
                return false;
            }
        }

        @Override
        public boolean shouldRunEveryTick() {
            return true;
        }

        @Override
        public void tick() {
            this.entity.getJumpControl().setActive();
        }
    }
}
