package net.voltexstudios.entity.custom;

import net.minecraft.entity.AnimationState;
import net.minecraft.entity.EntityType;
import net.minecraft.entity.ai.goal.*;
import net.minecraft.entity.attribute.DefaultAttributeContainer;
import net.minecraft.entity.attribute.EntityAttributes;
import net.minecraft.entity.mob.HostileEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.world.World;

public class MummyEntity extends HostileEntity {

    public final AnimationState idleAnimationState = new AnimationState();
    private int idleAnimationTimeout = 0;

    public MummyEntity(EntityType<? extends MummyEntity> entityType, World world) {
        super(entityType, world);
    }

    @Override
    protected void initGoals() {
        this.goalSelector.add(8, new LookAtEntityGoal(this, PlayerEntity.class, 8.0F));
        this.goalSelector.add(8, new LookAroundGoal(this));

        this.goalSelector.add(2, new MummyAttackGoal(this, 1f, false));
        this.goalSelector.add(7, new WanderAroundGoal(this, 0.7f));
        this.targetSelector.add(2, new ActiveTargetGoal<>(this, PlayerEntity.class, true));
    }

    @Override
    public void tick() {
        super.tick();

        if (this.getWorld().isClient()) {
            this.setupAnimationStates();
        }
    }

    private void setupAnimationStates() {
        if (this.idleAnimationTimeout <= 0) {
            this.idleAnimationTimeout = 40;
            this.idleAnimationState.start(this.age);
        } else {
            --this.idleAnimationTimeout;
        }
    }

    public static DefaultAttributeContainer.Builder createMummyAttributes() {
        return HostileEntity.createHostileAttributes()
                .add(EntityAttributes.GENERIC_MOVEMENT_SPEED, 0.30F)
                .add(EntityAttributes.GENERIC_FOLLOW_RANGE, 12.0)
                .add(EntityAttributes.GENERIC_MAX_HEALTH, 24.0)
                .add(EntityAttributes.GENERIC_ATTACK_DAMAGE, 5.0);
    }

    static class MummyAttackGoal extends MeleeAttackGoal {
        private final MummyEntity mummy;
        private int ticks;


        public MummyAttackGoal(MummyEntity mummy, double speed, boolean pauseWhenMobIdle) {
            super(mummy, speed, pauseWhenMobIdle);
            this.mummy = mummy;
        }

        @Override
        public void start() {
            super.start();
            this.ticks = 0;
        }

        @Override
        public void stop() {
            super.stop();
            this.mummy.setAttacking(false);
        }

        @Override
        public void tick() {
            super.tick();
            this.ticks++;
            this.mummy.setAttacking(this.ticks >= 5 && this.getCooldown() < this.getMaxCooldown() / 2);
        }
    }
}
